/*******************************************************************
  Simple Session Initiation Protocol Test Program for the Network Seminar
  Written by Yasusi Kanada on September 3, 2003
  Updated on September 10 and on October 21, 2003
*******************************************************************/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.InetAddress;

/**
 * Test Agent (contains a client and a server)
 */
public class Main extends JApplet implements SsipListener {

    /** Ports used for receiving/sending SSIP messages.
        (Port numbers are fixed.)			**/
    final int LocalPort = 8000;
    final int RemotePort = 8000;

    /** Remote site address **/
    String remoteAddress;

    /** SSIP stack provider **/
    SsipProvider ssipProvider;

    /** Window objects for input/output **/
    JTextField inputField = new JTextField();	// Address/message field
    JLabel messageLabel = new JLabel();		// Output message label

    /** Other buttons **/
    JButton callButton = new JButton(" Call ");
    JButton answerButton = new JButton("Answer");
    JButton sendButton = new JButton("Send");
    JButton hungButton = new JButton("Hung");

    /**
     * Constructor
     */
    public Main() {
        ssipProvider = new SsipProvider(SsipProvider.UseUDP,
                                        LocalPort, RemotePort, this);
		// instantiate a SsipProvider with UDP port #Port.
		// "UseUDP" can be replaced by "UseTCP".
        setGUI();
        updateButtons();
    }

    /**
     * Close this program.
     */
    void close() {
        ssipProvider.close();
    }

    /**
     * Send an INVITE message.
     */
    void invite(InetAddress remoteAddress) {
        this.remoteAddress = remoteAddress.getHostAddress();
        ssipProvider.invite(remoteAddress);
        messageLabel.setText("Calling: " + this.remoteAddress);
    }

    /**
     * Process an incoming INVITE message.
     */
    public void processInvite(InetAddress remoteAddress) {
        this.remoteAddress = remoteAddress.getHostAddress();
        messageLabel.setText("Ringing: " + this.remoteAddress);
        updateButtons();
    }

    /**
     * Send an OK message.
     */
    void ok() {
        if (ssipProvider.ok()) {
            messageLabel.setText("Connected to " + remoteAddress);
        } else {
            messageLabel.setText("Connection failed!");
        };
    }

    /**
     * Process an incoming OK message.
     */
    public void processOk() {
        messageLabel.setText("Connected to " + remoteAddress);
        updateButtons();
    }

    /**
     * Send a MESSAGE message.
     */
    void message(String message) {
        if (ssipProvider.message(message)) {
            messageLabel.setText("Message sent");
        } else {
            messageLabel.setText("Message sending failed");
        };
    }

    /**
     * Process an incoming MESSAGE message.
     */
    public void processMessage(String[] message) {
        messageLabel.setText("Message received: \"" +
                             (message.length > 1 ? message[1] : "") + "\"");
        updateButtons();
    }

    /**
     * Send a BYE message.
     */
    void bye() {
        if (ssipProvider.bye()) {
            messageLabel.setText("Hung up");
            remoteAddress = null;
        };
    }

    /**
     * Process an incoming BYE message.
     */
    public void processBye() {
        messageLabel.setText("Hung up");
        remoteAddress = null;
        updateButtons();
    }


    /**
     * Enable/disable buttons according to the current state.
     */
    void updateButtons() {
        if (ssipProvider == null) {
            return;
        };
        int state = ssipProvider.state;
        if (state == SsipProvider.Disconnected) {
            updateButtons1(true, false, false, false);
			// enables the call button only.
        } else if (state == SsipProvider.Calling) {
            updateButtons1(false, false, false, true);
			// enables the hung button only.
        } else if (state == SsipProvider.Ringing) {
            updateButtons1(false, true, false, true);
			// enables the answer and hung buttons.
        } else if (state == SsipProvider.Connected) {
            updateButtons1(false, false, true, true);
			// enables the send (-message) and hung buttons.
        };
    }

    void updateButtons1(boolean call, boolean answer,
                        boolean send, boolean hung) {
        callButton.setEnabled(call);
        answerButton.setEnabled(answer);
        sendButton.setEnabled(send);
        hungButton.setEnabled(hung);
    }

    /**
     * Initialize the window and window event handlers.
     */
    void setGUI() {
        JPanel panel = new JPanel(new FlowLayout());
        getContentPane().add(panel);

        JLabel inputLabel = new JLabel("Address/Message");
        panel.add(inputLabel);

        inputField.setPreferredSize(new Dimension(180, 30));
        panel.add(inputField);

        callButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
			// called when the call button is pushed.
                    callButton.setEnabled(false);
                    try {
                        InetAddress remoteAddress =
                            InetAddress.getByName(inputField.getText());
                        invite(remoteAddress);
                        inputField.setText("");
                    } catch (Exception x) {
                        x.printStackTrace();
                    };
                    updateButtons();
                }});
        panel.add(callButton);

        answerButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
			// called when the answer button is pushed.
                    answerButton.setEnabled(false);
                    ok();
                    updateButtons();
                }});
        panel.add(answerButton);

        sendButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
			// called when the send button is pushed.
                    sendButton.setEnabled(false);
                    message(inputField.getText());
                    inputField.setText("");
                    updateButtons();
                }});
        panel.add(sendButton);

        hungButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
			// called when the hung button is pushed.
                    hungButton.setEnabled(false);
                    bye();
                    updateButtons();
                }});
        panel.add(hungButton);

        panel.add(messageLabel);

    }

    /**
     * Main program
     */
    public static void main(String[] args) {
        final Main agent = new Main();
        JFrame window = new JFrame("SSIP Test Window");
        window.setSize(320, 130);
        window.getContentPane().add(agent);
        window.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
			// called when the close button is pushed.
                    agent.bye();
                    agent.close();
                    System.exit(0);
                }
            }
        );
        window.setVisible(true);
    }
}
