/*******************************************************************
  Simple Session Initiation Protocol (SSIP) Provider
  Written by Yasusi Kanada on September 2, 2003
  Updated on September 10, 2003
*******************************************************************/

import java.net.InetAddress;

/**
 * SSIP Stack Provider
 */
public class SsipProvider extends Thread {

    /** Lower layer: UDP or TCP **/
    public static final int UseUDP = 0;
    public static final int UseTCP = 1;

    /** Remote UDP/TCP port **/
    int remotePort;

    /** SSIP Listener (the user of this stack).
        (There must be only one user in this implementation.) **/
    SsipListener ssipListener;

    /** Lower network layer **/
    SocketIOStream stream;

    /** Possible SSIP stack states **/
    public static final int Disconnected = 0;
    public static final int Calling = 1;
    public static final int Ringing = 2;
    public static final int Connected = 3;

    /** Current SSIP stack state **/
    public int state = Disconnected;

    /** IP address of the remote site **/
    InetAddress remoteAddress;

    boolean done = false;

    /**
     * Constructor
     */
    public SsipProvider(int udpOrTcp, int localPort, int remotePort,
                        SsipListener ssipListener) {
        this.remotePort = remotePort;
        stream = udpOrTcp == 0
                     ? (SocketIOStream) new UDPSocketIOStream(localPort)
                     : (SocketIOStream) new TCPSocketIOStream(localPort);
        this.ssipListener = ssipListener;
        start();	// Start executing run() by a new thread.
    }

    /**
     * Close the SSIP stack.
     */
    public void close() {
        stream.close();
        done = true;
    }

    /**
     * Send a INVITE message.
     */
    public boolean invite(InetAddress remoteAddress) {
        if (state == Disconnected &&
            stream.write(remoteAddress, remotePort, "INVITE\n")) {
			// if sending INVITE message succeeded
            state = Calling;
            this.remoteAddress = remoteAddress;
            return true;
        };
        return false;
    }

    /**
     * Process an incoming INVITE message.
     */
    void processInvite(InetAddress fromAddress) {
        if (state == Disconnected) {
            state = Ringing;
            remoteAddress = fromAddress;
            ssipListener.processInvite(fromAddress);
        };
    }

    /**
     * Send an OK message.
     */
    public boolean ok() {
        if (state == Ringing && remoteAddress != null &&
            stream.write(remoteAddress, remotePort, "OK\n")) {
            state = Connected;
            return true;
        };
        return false;
    }

    /**
     * Process an incoming OK message.
     */
    void processOk(InetAddress fromAddress) {
        if (state == Calling &&
            fromAddress != null && fromAddress.equals(remoteAddress)) {
            state = Connected;
            ssipListener.processOk();
        };
    }

    /**
     * Send a MESSAGE message.
     */
    public boolean message(String message) {
        if (state == Connected && remoteAddress != null) {
            stream.write(remoteAddress, remotePort, "MESSAGE\n" + message + "\n");
            return true;
        };
        return false;
    }

    /**
     * Process an incoming MESSAGE message.
     */
    void processMessage(String[] message, InetAddress fromAddress) {
        if (state == Connected &&
            fromAddress != null && fromAddress.equals(remoteAddress)) {
            ssipListener.processMessage(message);
        };
    }

    /**
     * Send a BYE message.
     */
    public boolean bye() {
        if (state != Disconnected && remoteAddress != null &&
            stream.write(remoteAddress, remotePort, "BYE\n")) {
            remoteAddress = null;
            state = Disconnected;
            return true;
        };
        return false;
    }

    /**
     * Process an incoming BYE message.
     */
    void processBye(InetAddress fromAddress) {
        if (state != Disconnected &&
            fromAddress != null && fromAddress.equals(remoteAddress)) {
            state = Disconnected;
            remoteAddress = null;
            ssipListener.processBye();
        };
    }

    /**
     * Repeat waiting and processing incoming messages.
     */
    public void run() {
        while (!done) {
            String[] message = stream.read().split("\n");
		// separate lines (separate the message type and the content).
            InetAddress fromAddress = stream.getFromAddress();
            String type = message[0].trim();
            if (type.equals("INVITE")) {
                processInvite(fromAddress);
            } else if (type.equals("OK")) {
                processOk(fromAddress);
            } else if (type.equals("MESSAGE")) {
                processMessage(message, fromAddress);
            } else if (type.equals("BYE")) {
                processBye(fromAddress);
            } else {
                System.err.println("Error: unknown message type '" +
                                   type + "'");
            };
        };
    }
}
